@@ -36,7 +36,6 @@ CSTRING = 2275
 CSTRING_ARRAY = 1263
 DATE = 1082
 DATE_ARRAY = 1182
-DATETIME = 1114
 FLOAT = 701
 FLOAT_ARRAY = 1022
 INET = 869
@@ -45,6 +44,7 @@ INT2VECTOR = 22
 INTEGER = 23
 INTEGER_ARRAY = 1007
 INTERVAL = 1186
+INTERVAL_ARRAY = 1187
 OID = 26
 JSON = 114
 JSON_ARRAY = 199
@@ -69,7 +69,7 @@ STRING = 1043
 TEXT = 25
 TEXT_ARRAY = 1009
 TIME = 1083
-TIMEDELTA = 1186
+TIME_ARRAY = 1183
 TIMESTAMP = 1114
 TIMESTAMP_ARRAY = 1115
 TIMESTAMPTZ = 1184
@@ -143,52 +143,7 @@ def int_out(v):
     return str(v)
 
 
-def json_in(data):
-    return loads(data)
-
-
-def json_out(v):
-    return dumps(v)
-
-
-def null_out(v):
-    return None
-
-
-def numeric_in(data):
-    return Decimal(data)
-
-
-def numeric_out(d):
-    return str(d)
-
-
-def pginterval_in(data):
-    return PGInterval.from_str(data)
-
-
-def pginterval_out(v):
-    return str(v)
-
-
-def string_in(data):
-    return data
-
-
-def string_out(v):
-    return v
-
-
-def time_in(data):
-    pattern = "%H:%M:%S.%f" if "." in data else "%H:%M:%S"
-    return Datetime.strptime(data, pattern).time()
-
-
-def time_out(v):
-    return v.isoformat()
-
-
-def timedelta_in(data):
+def interval_in(data):
     t = {}
 
     curr_val = None
@@ -210,7 +165,7 @@ def timedelta_in(data):
     return Timedelta(**t)
 
 
-def timedelta_out(v):
+def interval_out(v):
     return " ".join(
         (
             str(v.days),
@@ -223,6 +178,51 @@ def timedelta_out(v):
     )
 
 
+def json_in(data):
+    return loads(data)
+
+
+def json_out(v):
+    return dumps(v)
+
+
+def null_out(v):
+    return None
+
+
+def numeric_in(data):
+    return Decimal(data)
+
+
+def numeric_out(d):
+    return str(d)
+
+
+def pg_interval_in(data):
+    return PGInterval.from_str(data)
+
+
+def pg_interval_out(v):
+    return str(v)
+
+
+def string_in(data):
+    return data
+
+
+def string_out(v):
+    return v
+
+
+def time_in(data):
+    pattern = "%H:%M:%S.%f" if "." in data else "%H:%M:%S"
+    return Datetime.strptime(data, pattern).time()
+
+
+def time_out(v):
+    return v.isoformat()
+
+
 def timestamp_in(data):
     if data in ("infinity", "-infinity"):
         return data
@@ -515,10 +515,12 @@ cidr_array_in = _array_in(cidr_in)
 date_array_in = _array_in(date_in)
 inet_array_in = _array_in(inet_in)
 int_array_in = _array_in(int)
+interval_array_in = _array_in(interval_in)
 json_array_in = _array_in(json_in)
 float_array_in = _array_in(float)
 numeric_array_in = _array_in(numeric_in)
 string_array_in = _array_in(string_in)
+time_array_in = _array_in(time_in)
 timestamp_array_in = _array_in(timestamp_in)
 timestamptz_array_in = _array_in(timestamptz_in)
 uuid_array_in = _array_in(uuid_in)
@@ -551,6 +553,7 @@ PG_ARRAY_TYPES = {
     DATE: DATE_ARRAY,  # date[]
     FLOAT: FLOAT_ARRAY,  # float8[]
     INET: INET_ARRAY,  # inet[]
+    INTERVAL: INTERVAL_ARRAY,  # interval[]
     INTEGER: INTEGER_ARRAY,  # int4[]
     JSON: JSON_ARRAY,  # json[]
     JSONB: JSONB_ARRAY,  # jsonb[]
@@ -558,6 +561,7 @@ PG_ARRAY_TYPES = {
     NUMERIC: NUMERIC_ARRAY,  # numeric[]
     SMALLINT: SMALLINT_ARRAY,  # int2[]
     TEXT: TEXT_ARRAY,  # text[]
+    TIME: TIME_ARRAY,  # time[]
     TIMESTAMP: TIMESTAMP_ARRAY,  # timestamp[]
     TIMESTAMPTZ: TIMESTAMPTZ_ARRAY,  # timestamptz[]
     UUID_TYPE: UUID_ARRAY,  # uuid[]
@@ -573,26 +577,23 @@ def inspect_datetime(value):
         return PY_TYPES[TIMESTAMPTZ]
 
 
-min_int2, max_int2 = -(2 ** 15), 2 ** 15
-min_int4, max_int4 = -(2 ** 31), 2 ** 31
-min_int8, max_int8 = -(2 ** 63), 2 ** 63
+def int_oid(value):
+    oid = NUMERIC
+
+    if MIN_INT2 < value < MAX_INT2:
+        oid = SMALLINT
+
+    elif MIN_INT4 < value < MAX_INT4:
+        oid = INTEGER
+
+    elif MIN_INT8 < value < MAX_INT8:
+        oid = BIGINT
+
+    return oid
 
 
 def inspect_int(value):
-    if min_int2 < value < max_int2:
-        return PY_TYPES[SMALLINT]
-    if min_int4 < value < max_int4:
-        return PY_TYPES[INTEGER]
-    if min_int8 < value < max_int8:
-        return PY_TYPES[BIGINT]
-    return PY_TYPES[Decimal]
-
-
-def array_find_first_element(arr):
-    for v in array_flatten(arr):
-        if v is not None:
-            return v
-    return None
+    return PY_TYPES[int_oid(value)]
 
 
 def array_flatten(arr):
@@ -606,13 +607,36 @@ def array_flatten(arr):
 
 def array_inspect(array):
     first_element = None
-    for v in array_flatten(array):
+    flattened = list(array_flatten(array))
+
+    for v in flattened:
         if v is not None:
             first_element = v
             break
 
     if first_element is None:
         oid = VARCHAR
+
+    elif isinstance(first_element, bool):
+        oid = BOOLEAN
+
+    elif isinstance(first_element, int):
+        int_oids = [SMALLINT, INTEGER, BIGINT, NUMERIC]
+        for v in flattened:
+            if v is None:
+                continue
+
+            v_oid = int_oid(v)
+            try:
+                oid_idx = int_oids.index(v_oid)
+            except ValueError:
+                continue
+
+            if oid_idx > 0:
+                int_oids = int_oids[oid_idx:]
+
+        oid = int_oids[0]
+
     else:
         oid, _ = make_param(PY_TYPES, first_element)
 
@@ -657,7 +681,9 @@ date_array_out = _array_out(date_out)
 float_array_out = _array_out(float_out)
 inet_array_out = _array_out(inet_out)
 int_array_out = _array_out(int_out)
+interval_array_out = _array_out(interval_out)
 numeric_array_out = _array_out(numeric_out)
+time_array_out = _array_out(time_out)
 timestamp_array_out = _array_out(timestamp_out)
 timestamptz_array_out = _array_out(timestamptz_out)
 uuid_array_out = _array_out(uuid_out)
@@ -735,13 +761,16 @@ PY_TYPES = {
     INET_ARRAY: (INET_ARRAY, inet_array_out),  # inet[]
     INTEGER: (INTEGER, int_out),  # int4
     INTEGER_ARRAY: (INTEGER_ARRAY, int_array_out),  # int4[]
+    INTERVAL_ARRAY: (INTERVAL_ARRAY, interval_array_out),  # interval[]
     JSON_ARRAY: (JSON_ARRAY, json_array_out),  # json[]
     JSONB_ARRAY: (JSONB_ARRAY, json_array_out),  # jsonb[]
     MONEY: (MONEY, string_array_out),  # money[]
     MONEY_ARRAY: (MONEY_ARRAY, numeric_array_out),  # money[]
+    NUMERIC: (NUMERIC, numeric_out),  # numeric
     NUMERIC_ARRAY: (NUMERIC_ARRAY, numeric_array_out),  # numeric[]
     SMALLINT: (SMALLINT, int_out),  # int2
     SMALLINT_ARRAY: (SMALLINT_ARRAY, int_array_out),  # int2[]
+    TIME_ARRAY: (TIME_ARRAY, time_array_out),  # time[]
     TIMESTAMP: (TIMESTAMP, timestamp_out),  # timestamp
     TIMESTAMP_ARRAY: (TIMESTAMP_ARRAY, timestamp_array_out),  # timestamp[]
     TIMESTAMPTZ: (TIMESTAMPTZ, timestamptz_out),  # timestamptz
@@ -749,15 +778,15 @@ PY_TYPES = {
     UUID_ARRAY: (UUID_ARRAY, uuid_array_out),  # uuid
     VARCHAR_ARRAY: (VARCHAR_ARRAY, string_array_out),  # varchar[]
     Date: (DATE, date_out),  # date
-    Decimal: (1700, numeric_out),  # numeric
+    Decimal: (NUMERIC, numeric_out),  # numeric
     Enum: (UNKNOWN, enum_out),  # enum
     IPv4Address: (INET, inet_out),  # inet
     IPv6Address: (INET, inet_out),  # inet
     IPv4Network: (INET, inet_out),  # inet
     IPv6Network: (INET, inet_out),  # inet
-    PGInterval: (1186, pginterval_out),  # interval
+    PGInterval: (1186, interval_out),  # interval
     Time: (TIME, time_out),  # time
-    Timedelta: (1186, timedelta_out),  # interval
+    Timedelta: (1186, interval_out),  # interval
     UUID: (UUID_TYPE, uuid_out),  # uuid
     bool: (BOOLEAN, bool_out),  # bool
     bytearray: (BYTES, bytes_out),  # bytea
@@ -801,6 +830,8 @@ PG_TYPES = {
     NUMERIC: numeric_in,  # numeric
     NUMERIC_ARRAY: numeric_array_in,  # numeric[]
     OID: int,  # oid
+    INTERVAL: interval_in,  # interval
+    INTERVAL_ARRAY: interval_array_in,  # interval[]
     REAL: float,  # float4
     REAL_ARRAY: float_array_in,  # float4[]
     SMALLINT: int,  # int2
@@ -809,7 +840,8 @@ PG_TYPES = {
     TEXT: string_in,  # text
     TEXT_ARRAY: string_array_in,  # text[]
     TIME: time_in,  # time
-    TIMEDELTA: timedelta_in,  # interval
+    TIME_ARRAY: time_array_in,  # time[]
+    INTERVAL: interval_in,  # interval
     TIMESTAMP: timestamp_in,  # timestamp
     TIMESTAMP_ARRAY: timestamp_array_in,  # timestamp
     TIMESTAMPTZ: timestamptz_in,  # timestamptz
