@@ -32,7 +32,6 @@ from pg8000.converters import (
     TEXT,
     TEXT_ARRAY,
     TIME,
-    TIMEDELTA,
     TIMESTAMP,
     TIMESTAMPTZ,
     UNKNOWN,
@@ -40,7 +39,10 @@ from pg8000.converters import (
     VARCHAR,
     VARCHAR_ARRAY,
     XID,
+    interval_in as timedelta_in,
     make_params,
+    pg_interval_in as pginterval_in,
+    pg_interval_out as pginterval_out,
 )
 from pg8000.core import CoreConnection
 from pg8000.dbapi import (
@@ -103,6 +105,7 @@ DATETIME = TIMESTAMP
 NUMBER = DECIMAL = NUMERIC
 DECIMAL_ARRAY = NUMERIC_ARRAY
 ROWID = OID
+TIMEDELTA = INTERVAL
 
 
 def connect(
@@ -739,64 +742,67 @@ class PreparedStatement:
 
 
 __all__ = [
-    Warning,
-    DataError,
-    DatabaseError,
-    connect,
-    InterfaceError,
-    ProgrammingError,
-    Error,
-    OperationalError,
-    IntegrityError,
-    InternalError,
-    NotSupportedError,
-    Connection,
-    Cursor,
-    Binary,
-    Date,
-    DateFromTicks,
-    Time,
-    TimeFromTicks,
-    Timestamp,
-    TimestampFromTicks,
-    BINARY,
-    PGInterval,
-    STRING,
-    NUMBER,
-    DATETIME,
-    TIME,
-    BOOLEAN,
-    INTEGER,
-    BIGINTEGER,
-    INTERVAL,
-    JSON,
-    JSONB,
-    UNKNOWN,
-    NULLTYPE,
-    ROWID,
-    BOOLEAN_ARRAY,
-    BYTES,
-    CHAR,
-    CHAR_ARRAY,
-    DATE,
-    DECIMAL,
-    DECIMAL_ARRAY,
-    FLOAT,
-    FLOAT_ARRAY,
-    INET,
-    INT2VECTOR,
-    INTEGER_ARRAY,
-    MACADDR,
-    NAME,
-    NAME_ARRAY,
-    OID,
-    TEXT,
-    TEXT_ARRAY,
-    TIMEDELTA,
-    TIMESTAMP,
-    TIMESTAMPTZ,
-    UUID_TYPE,
-    VARCHAR,
-    VARCHAR_ARRAY,
-    XID,
+    "BIGINTEGER",
+    "BINARY",
+    "BOOLEAN",
+    "BOOLEAN_ARRAY",
+    "BYTES",
+    "Binary",
+    "CHAR",
+    "CHAR_ARRAY",
+    "Connection",
+    "Cursor",
+    "DATE",
+    "DATETIME",
+    "DECIMAL",
+    "DECIMAL_ARRAY",
+    "DataError",
+    "DatabaseError",
+    "Date",
+    "DateFromTicks",
+    "Error",
+    "FLOAT",
+    "FLOAT_ARRAY",
+    "INET",
+    "INT2VECTOR",
+    "INTEGER",
+    "INTEGER_ARRAY",
+    "INTERVAL",
+    "IntegrityError",
+    "InterfaceError",
+    "InternalError",
+    "JSON",
+    "JSONB",
+    "MACADDR",
+    "NAME",
+    "NAME_ARRAY",
+    "NULLTYPE",
+    "NUMBER",
+    "NotSupportedError",
+    "OID",
+    "OperationalError",
+    "PGInterval",
+    "ProgrammingError",
+    "ROWID",
+    "STRING",
+    "TEXT",
+    "TEXT_ARRAY",
+    "TIME",
+    "TIMEDELTA",
+    "TIMESTAMP",
+    "TIMESTAMPTZ",
+    "Time",
+    "TimeFromTicks",
+    "Timestamp",
+    "TimestampFromTicks",
+    "UNKNOWN",
+    "UUID_TYPE",
+    "VARCHAR",
+    "VARCHAR_ARRAY",
+    "Warning",
+    "XID",
+    "connect",
+    "pginterval_in",
+    "pginterval_out",
+    "timedelta_in",
 ]
